﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Telerik.Web.UI;

namespace TestApplication
{
    public partial class Default : System.Web.UI.Page
    {
        // RadComboBox1 is bound to a collection of Category projection instances.
        protected void RadComboBox1_ItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            // A reflection is used to find the "CategoryName" property and to assign "NewCategory" to it.
            Default.SetItemAttribute(e.Item.DataItem, "CategoryName", "NewCategory");
        }

        // RadComboBox2 is bound to a CategoryView collection of instances. The CategoryView represents exactly the projection from the scenation1 - RadComboBox1
        protected void RadComboBox2_ItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            CategoryView instance = e.Item.DataItem as CategoryView;
            if (instance != null)
            {
                instance.CategoryName = "NewCategory";
            }
        }

        public static void SetItemAttribute(object instance, string attributeName, object attributeValue)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }

            if (string.IsNullOrEmpty(attributeName))
            {
                throw new ArgumentNullException("attributeName");
            }

            PropertyInfo[] properties = instance.GetType().GetProperties(BindingFlags.Public | BindingFlags.Instance);
            if (properties == null || properties.Length == 0)
            {
                return;
            }

            PropertyInfo property = properties.FirstOrDefault(x => string.Equals(attributeName, x.Name));
            if (property == null)
            {
                string errorMessage = string.Format("There is property with the specified name: {0}.", attributeName);

                throw new KeyNotFoundException(errorMessage);
            }

            if (!property.CanWrite)
            {
                string errorMessage = string.Format("The property '{0}' does not have setter specified.", attributeName);

                throw new InvalidOperationException(errorMessage);
            }

            property.SetValue(instance, attributeValue, null);
        }
    }
}